(**************************************************************************)
(*                                                                        *)
(*  This file is part of Frama-C.                                         *)
(*                                                                        *)
(*  Copyright (C) 2007-2009                                               *)
(*    CEA (Commissariat  l'nergie Atomique)                             *)
(*                                                                        *)
(*  you can redistribute it and/or modify it under the terms of the GNU   *)
(*  Lesser General Public License as published by the Free Software       *)
(*  Foundation, version 2.1.                                              *)
(*                                                                        *)
(*  It is distributed in the hope that it will be useful,                 *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU Lesser General Public License for more details.                   *)
(*                                                                        *)
(*  See the GNU Lesser General Public License version 2.1                 *)
(*  for more details (enclosed in the file licenses/LGPLv2.1).            *)
(*                                                                        *)
(**************************************************************************)

(** Frama-C Main.
    @plugin development guide *)

let obfuscate_code code_fmt files =
  let dictionary = Obfuscate.obfuscate files in
  Format.fprintf code_fmt "// Start of dictionary for obfuscation:@\n";
  Hashtbl.iter
    (fun k v -> Format.fprintf code_fmt "#define %s %s@\n" k v)
    dictionary;
  Format.fprintf code_fmt "// End of dictionary for obfuscation.@\n";
  Format.fprintf code_fmt "@[%a@]" (Cil.d_file (new Printer.print())) files

let all_plugins () =
  if Parameters.TypeCheck.get () then ignore (Ast.get());
  if Parameters.Obfuscate.get () then begin
    obfuscate_code (Parameters.CodeOutput.get_fmt ()) (Ast.get ());
    exit 0
  end;
  (* Printing files before anything else (in debug mode only) *)
  if Parameters.Debug.get () > 0 then File.pretty ();
  (* Syntactic constant folding before analysing files if required *)
  if Parameters.Constfold.get () then
    Cil.visitCilFileSameGlobals (Cil.constFoldVisitor true) (Ast.get ());
  try
    Dynamic.Main.apply (); (* for Helium-compatibility purpose only *)
    Db.Main.apply ();
    (* Printing code if required, have to be done at end *)
    if Parameters.PrintCode.get () then File.pretty ();
    Cilutil.flush_all ();
  with Globals.No_such_entry_point msg ->
    Kernel.error "%s" msg

let global_toplevel_startup_hook () =
  try
    all_plugins ()
  with Sys.Break as e ->
    let forcevalues = Parameters.Dynamic.Bool.get "-val" in
    let forceout = Parameters.Dynamic.Bool.get "-out" in
    let forcedeps = Parameters.Dynamic.Bool.get "-deps" in
    Kernel.warning "user interrupted computations; \
dumping whatever we may have at this point.";
    if forcevalues || forcedeps || forceout then
	Globals.Functions.iter
          (fun kf ->
	     if Kernel_function.is_definition kf then begin
	       if forceout then
		 !Db.Outputs.display Format.std_formatter kf;
	       if forcevalues then
		 Db.Value.display Format.std_formatter kf;
             end);
    raise e

let () =
  Cmdline.run_as_the_main global_toplevel_startup_hook;
  (* [JS 2009/04/11] should try to load files when replaying *)
  Db.Toplevel.replay := global_toplevel_startup_hook;
  Db.Toplevel.run_all_plugins := all_plugins;

(*
Local Variables:
compile-command: "LC_ALL=C make -C ../.."
End:
*)
